#!/bin/bash
# Script to fetch package sources

if [ -r "$PLATFORM_LOC/scripts/utils.sh" ]; then
	source "$PLATFORM_LOC/scripts/utils.sh"
else
	source "$script_lib_dir/utils.sh"
fi

targetDir=$DOWNLOAD_LOC
srcSha256=$SRC_SHA256
srcMd5=$SRC_MD5

srcUrl=$SRC_LOC

# For relative URL, add the workspace location in prefix
case $srcUrl in
	*:*)
		;;
	/*)
		;;
	empty)
		;;	
	*)
		srcUrl=$WORKSPACE_LOC/$srcUrl
		;;
esac

srcFilename=${srcUrl##*/}
srcFile="$targetDir/$srcFilename"

require_script_once scms_utils.sh
require_script_once mirrors_utils.sh

gen_sha256() {
	local src="$1"
	local computed_sha256=$(sha256sum -b "$src")
	echo ${computed_sha256%% *}
}

gen_md5() {
	local src="$1"
	local computed_md5=$(md5sum -b "$src")
	echo ${computed_md5%% *}
}

gen_sha256_dir() {
	local src="$1"
	local sumfile=/tmp/sha256file.$$
	local fil
	sha256sum -b $(find $src -type f | sort | uniq) > $sumfile
	local computed_sha256=$(sha256sum -b "$sumfile")
	echo ${computed_sha256%% *}
}

gen_md5_dir() {
	local src="$1"
	local sumfile=/tmp/sha256file.$$
	local fil
	md5sum -b $(find $src -type f | sort | uniq) > $sumfile
	local computed_md5=$(md5sum -b "$sumfile")
	echo ${computed_md5%% *}
}
	
check_downloaded_file() {
	local srcFile="$1"
	local cksumerror=${2:-false}
	local computed_sha256 computed_md5 file_sha256
	
	if [ -n "$scms_tool" ]; then
		if ${scms_tool}_check "$srcUrl" "$targetDir" "$srcFilename"; then
			return 0
		else
			return 1
		fi
	fi
		
	case $srcUrl in
		http://* | https://* | ftp://* | file://*)
			if [ ! -r "$srcFile" ]; then
				return 1
			fi
			computed_sha256=$(gen_sha256 "$srcFile")
			computed_md5=$(gen_md5 "$srcFile")
			if [ -n "$srcSha256" ]; then
				if [ "$srcSha256" != "$computed_sha256" ]; then
					rm -f $srcFile.done
					if $cksumerror; then
						fatal 1 "Sha256 checksum for '$srcFile' is wrong: expected '$srcSha256', found '$computed_sha256'"
					else
						warning "Found '$srcFile' but sha256 checksum is wrong: expected '$srcSha256', found '$computed_sha256'"
						return 1
					fi
				fi
			else
				warning "No sha256 checksum provided for '$srcUrl'; think at adding one for security"
				warning "Sha256 checksum for '$srcUrl' should be '$computed_sha256'"
			fi
			if [ -n "$srcMd5" ]; then
				if [ "$srcMd5" != "$computed_md5" ]; then
					rm -f $srcFile.done
					if $cksumerror; then
						fatal 1 "Md5 checksum for '$srcFile' is wrong: expected '$srcMd5', found '$computed_md5'"
					else
						warning "Found '$srcFile' but md5 checksum is wrong: expected '$srcMd5', found '$computed_md5'"
						return 1
					fi
				fi
			else
				warning "No md5 checksum provided for '$srcUrl'; think at adding one for security"
				warning "Md5 checksum for '$srcUrl' should be '$computed_md5'"
			fi
			(echo "MD5=$computed_md5"; echo "SHA256=$computed_sha256") > $srcFile.done
			(echo "MD5=$computed_md5"; echo "SHA256=$computed_sha256") > $(recipe_task_state_file).outstamp
			;;
		*:*)
			rm -f $srcFile.done
			rm -f $(recipe_task_state_file).outstamp
			remove_task_output_state
			return 1
			;;
		*)
			if [ -d "$srcUrl" ]; then
				# Local directory, typically an Eclipse project in the workspace
				original_sha256=$(cat "$srcFile.done" 2> /dev/null)
				computed_sha256=$(gen_sha256_dir "$srcUrl")
				echo $computed_sha256 > "$srcFile.done"
				if [ "$computed_sha256" = "$original_sha256" ]; then
					for fil in "$srcFile-$PACKAGE_VERSION.tar".* "$srcFile.tar".*; do
						if [ "$fil" != "${fil%.done}" ]; then continue; fi
						if [ -r "$fil" ]; then
							file_sha256=$(cat "$fil.src-sha256" 2> /dev/null)
							if [ "$file_sha256" = "$computed_sha256" ]; then
								computed_sha256=$(gen_sha256 "$fil")
								computed_md5=$(gen_md5 "$fil")
								(echo "MD5=$computed_md5"; echo "SHA256=$computed_sha256") > $fil.done
								(echo "MD5=$computed_md5"; echo "SHA256=$computed_sha256") > $(recipe_task_state_file).outstamp
								return 0
							else
								rm -f "$fil" "$fil.src-sha256"
							fi
						fi
					done
				fi
				rm -f $(recipe_task_state_file).outstamp
				return 1
			else
				# Local tarball
				for fil in "$srcFile" "$srcFile-$PACKAGE_VERSION" "$srcFile-$PACKAGE_VERSION.tar".* "$srcFile.tar".*; do
					if [ -r "$fil" ]; then
						computed_sha256=$(gen_sha256 "$fil")
						computed_md5=$(gen_md5 "$fil")
						(echo "MD5=$computed_md5"; echo "SHA256=$computed_sha256") > $fil.done
						(echo "MD5=$computed_md5"; echo "SHA256=$computed_sha256") > $(recipe_task_state_file).outstamp
						return 0
					fi
				done
			fi
			rm -f $srcFile.done
			rm -f $(recipe_task_state_file).outstamp
			return 1
			;;
	esac
}

wget_fetch() {
	local destDir=$1
	local srcUrl=$2
	
	# Unset LD_LIBRATY_PATH to avoid error with sysroot OPENSSL
	unset LD_LIBRARY_PATH
	
	# Check original URL
	# We accept bad certificates for https URIs,
	# as anyway we'll check the fetched file MD5 and SHA256
	log "Check URL: $srcUrl"
	wget -q --spider --no-check-certificate "$srcUrl" > /dev/null 2>&1
	if [ "$?" -ne 0 ]; then
		log " -> Cannot access $srcUrl"

		# Check mirrors URL
		local listMirrorUrls=$(get_mirror_urls "$srcUrl")
		for mirrorUrl in $listMirrorUrls ; do
			log "Check mirror URL: $mirrorUrl"
			wget --spider --no-check-certificate "$mirrorUrl" > /dev/null 2>&1
			
			# If mirror URL is valid
			if [ "$?" -eq 0 ]; then
				log "Valid mirror URL: $mirrorUrl"
				wget_download $destDir $mirrorUrl
				exit $?
			fi
		done
	fi
	
	# Download original URL
	wget_download $destDir $srcUrl
}

wget_download() {
	local destDir=$1
	local srcUrl=$2
	
	# Unset LD_LIBRATY_PATH to avoid error with native sysroot OPENSSL, use host OPENSSL
	# Not sure this is still needed, or even effective, as we patched DT-RPATH/DT_RUNPATH
	# in native executables...
	unset LD_LIBRARY_PATH
	
	# Don't check HTTPS certificate: we will check the downloaded file MD5/SHA256 signatures
	wget --no-check-certificate -nv -nc -P "$destDir" "$srcUrl"
}

do_buildstep_prepend

if check_downloaded_file "$srcFile"; then
	log "'$srcFilename' has been found in '$targetDir'"
elif [ -n "$scms_tool" ]; then
	if ${scms_tool}_fetch "$srcUrl" "$targetDir" "$srcFilename"; then
		true
	else
		exit 1
	fi
else
	case $srcUrl in
		http://* | https://* | ftp://*)
			log "Downloading $srcUrl ... into $targetDir"
			rm -f "$srcFile" # Just in case the file exist but is not readable
			
			wget_fetch $targetDir $srcUrl
			check_downloaded_file "$srcFile" true
			;;
		file://*)
			rm -f "$srcFile" # Just in case the file exist but is not readable
			srcPath=${srcUrl#file://}
			case $srcPath in
				/*)
					log "Searching $srcPath on local machine"
					srcDir=""
					;;
				*)
					log "Searching $srcPath in recipe resources"
					srcDir="$recipe_metadatadir/resources/"
					;;
			esac
			if [ -r "${srcDir}$srcPath" ]; then
				cp "${srcDir}$srcPath" "$srcFile"
				check_downloaded_file "$srcFile" true
			else
				fatal 1 "Resource file '$srcPath' not found"
			fi
			;;
		*:*)
			rm -f $srcFile.done
			remove_task_output_state
			fatal 1 "Unsupported scheme: '${srcUrl%%:*}:'"
			;;
		*)
			if [ "$srcUrl" = "" ]; then
				fatal 1 "No source URL to fetch from..."
			fi
			if [ "$srcUrl" = "empty" ]; then
				srcDir="/tmp/fetch.$$"
				srcFilename="$srcUrl"
				srcUrl="$srcDir/$srcFilename"
				mkdir -p "$srcUrl"
			fi
			if [ -d "$srcUrl" ]; then
				# Local directory, typically an Eclipse project in the workspace
				if [ "$PACKAGE_VERSION" != "none" ]; then
					srcFile=${srcFile%-$PACKAGE_VERSION}-$PACKAGE_VERSION
				fi
				srcDir=${srcUrl%/*}
				rm -f "$srcFile.tar".*
				# Note that we do not pack the _build directory, that contain build history and will be
				# preserved when re-importing in the same workspace but is irrelevant in another workspace
				log "Packing '$srcUrl' in '$srcFile.tar.xz'"
				log_exec tar cJf "$srcFile.tar.xz" -C "$srcDir" --exclude=_build "$srcFilename"
				# Beware that src-sha256 is computed before adding the version 
				cp "$(dirname $srcFile)/$srcFilename.done" "$srcFile.tar.xz.src-sha256"
				computed_sha256="$(gen_sha256 "$srcFile.tar.xz")"
				computed_md5="$(gen_md5 "$srcFile.tar.xz")"
				(echo "MD5=$computed_md5"; echo "SHA256=$computed_sha256") > $srcFile.tar.xz.done
				(echo "MD5=$computed_md5"; echo "SHA256=$computed_sha256") > $(recipe_task_state_file).outstamp
			elif [ -r "$srcUrl" ]; then
				# Local tarball
				log "Copying '$srcUrl' in '$targetDir'"
				log_exec cp "$srcUrl" "$targetDir"
				computed_sha256="$(gen_sha256 "$srcFile")"
				computed_md5="$(gen_md5 "$srcFile")"
				(echo "MD5=$computed_md5"; echo "SHA256=$computed_sha256") > $srcFile.done
				(echo "MD5=$computed_md5"; echo "SHA256=$computed_sha256") > $(recipe_task_state_file).outstamp
			else
				fatal 2 "Can't find '$srcUrl'"
			fi
			;;
	esac
fi

do_buildstep_append